/*
 *  Implementation of COSS Relationship Service for MICO
 *  Copyright (C) 1998 Karel Gardas with the assistance of Kay Roemer
 *                                                       & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 *  or to my private e-mail:
 *                 gardask@alpha.inf.upol.cz
 */


#ifndef __RoleFactory_impl_h__
#define __RoleFactory_impl_h__

#include <mico/CosRelationships.h>
#include <mico/Role_impl.h>


//#define DEBUG 1

extern int counter;

template<class T>
class RoleFactory_impl : virtual public CosRelationships::RoleFactory_skel
{
  CORBA::InterfaceDef_ptr r_type;
  //CORBA::InterfaceDef_ptr rel_type;
  CORBA::Boolean first;
  CosRelationships::InterfaceDefs* i_defs;
  CORBA::Boolean InterfaceDefs_nedef;
  CORBA::Long max_card;
  CORBA::Long min_card;
  char* iterator_server;
  char* rel_ship_repoid;
  char* name_of_role_server;
public:
  RoleFactory_impl ();
  RoleFactory_impl (CORBA::Long max, CORBA::Long min);
  RoleFactory_impl (CORBA::Long max, CORBA::Long min, 
		    CosRelationships::InterfaceDefs* 
		    related_objects_interfaces);

  RoleFactory_impl (CORBA::Long max, CORBA::Long min,
                    CosRelationships::InterfaceDefs* 
		    related_objects_interfaces,
		    char*);

  RoleFactory_impl (const CORBA::BOA::ReferenceData &refdata,
		    CORBA::Long max, CORBA::Long min,
                    CosRelationships::InterfaceDefs*
                    related_objects_interfaces);

  RoleFactory_impl (const CORBA::BOA::ReferenceData &refdata,
		    CORBA::Long max, CORBA::Long min,
                    CosRelationships::InterfaceDefs*
                    related_objects_interfaces,
                    char*);

  RoleFactory_impl (const CORBA::BOA::ReferenceData &refdata,
		    CORBA::Long max, CORBA::Long min,
                    CosRelationships::InterfaceDefs*
                    related_objects_interfaces,
		    char* iterator,
		    char* name);

  RoleFactory_impl (const CORBA::BOA::ReferenceData &refdata,
		    CORBA::Long max, CORBA::Long min,
                    CosRelationships::InterfaceDefs*
                    related_objects_interfaces,
		    char* iterator,
		    char* rel_ship_repoid,
		    char* name);


  virtual CORBA::InterfaceDef_ptr role_type ();
  virtual CORBA::ULong max_cardinality ();
  virtual CORBA::ULong min_cardinality ();
  virtual CosRelationships::InterfaceDefs* related_object_types();
  virtual CosRelationships::Role_ptr create_role( 
 		           CosRelationships::RelatedObject_ptr related_object);

  virtual Role_impl* create (CosRelationships::RelatedObject_ptr);
  virtual void destroy ();
// extension
  CORBA::ImplementationDef_ptr find_impl (const char *name);
     
};



template <class T>
RoleFactory_impl<T>::RoleFactory_impl()
{
#if DEBUG
  cout << "<RoleFactory_impl> RoleFactory constructor..." << flush;
#endif
  iterator_server = CORBA::string_dup ("");
  name_of_role_server = CORBA::string_dup ("");
  rel_ship_repoid = CORBA::string_dup ("");
  first = TRUE;
  InterfaceDefs_nedef = TRUE;
  i_defs = new CosRelationships::InterfaceDefs;
  i_defs->length(0);
#if DEBUG
  cout << "done.\n";
#endif
}


template <class T>
RoleFactory_impl<T>::RoleFactory_impl(CORBA::Long max, CORBA::Long min)
{
#if DEBUG
  cout << "  <RoleFactory_impl> RoleFactory_impl (max, min)" << flush;
#endif
  first = TRUE;
  rel_ship_repoid = CORBA::string_dup ("");
  InterfaceDefs_nedef = TRUE;
  i_defs = new CosRelationships::InterfaceDefs;
  i_defs->length(0);
  max_card = max;
  min_card = min;
#if DEBUG
  cout << "done.\n";
#endif
}



template <class T>
RoleFactory_impl<T>::RoleFactory_impl
(CORBA::Long max, CORBA::Long min,
 CosRelationships::InterfaceDefs* related_objects_interfaces)
{
#if DEBUG
  cout << "  <RoleFactory_impl> RoleFactory_impl "
       << "(max, min, related_objects_interfaces) ..." << flush;
#endif
  first = TRUE;
  rel_ship_repoid = CORBA::string_dup ("");
  InterfaceDefs_nedef = FALSE;
  i_defs = related_objects_interfaces;

#if DEBUG
  cout << "  <RoleFactory_impl> Describe interfaces of related "
       << i_defs->length () << " objects:\n";
  for (CORBA::ULong i=0; i<i_defs->length (); i++) {
    cout << "  <RoleFactory_impl> " << i+1 << ". " << flush;
    CORBA::InterfaceDef::FullInterfaceDescription_var full =
      (*i_defs)[i]->describe_interface ();
    //cout << "Full description:\n";
    //cout << "name: " << full->name << "\n";
    cout << full->id << "\n";
    //cout << "defined_in: " << full->defined_in << "\n";
    //cout << "defined_in: " << full->defined_in << "\n";
    //cout << "version: "<< full->version << "\n";
    //cout << "type code: " << full->type << "\n";
  }
#endif
  max_card = max;
  min_card = min;
#if DEBUG
  cout << "  <RoleFactory_impl> RoleFactory_impl (...) done.\n";
#endif
}


template <class T>
RoleFactory_impl<T>::RoleFactory_impl
(CORBA::Long max, CORBA::Long min,
 CosRelationships::InterfaceDefs* related_objects_interfaces,
 char* relship_repoid)
{
#if DEBUG
  cout << "  <RoleFactory_impl> RoleFactory_impl "
       << "(max, min, related_objects_interfaces) ..." << flush;
#endif
  first = TRUE;
  InterfaceDefs_nedef = FALSE;
  i_defs = related_objects_interfaces;
  rel_ship_repoid = CORBA::string_dup (relship_repoid);

#if DEBUG
  cout << "  <RoleFactory_impl> Describe interfaces of related "
       << i_defs->length () << " objects:\n";
  for (CORBA::ULong i=0; i<i_defs->length (); i++) {
    cout << "  <RoleFactory_impl> " << i+1 << ". " << flush;
    CORBA::InterfaceDef::FullInterfaceDescription_var full =
      (*i_defs)[i]->describe_interface ();
    //cout << "Full description:\n";
    //cout << "name: " << full->name << "\n";
    cout << full->id << "\n";
    //cout << "defined_in: " << full->defined_in << "\n";
    //cout << "version: "<< full->version << "\n";
    //cout << "type code: " << full->type << "\n";
  }
#endif
  max_card = max;
  min_card = min;

#if 0
  return CORBA::InterfaceDef::_duplicate (r_type);
#endif
}


template <class T>
RoleFactory_impl<T>::RoleFactory_impl
(const CORBA::BOA::ReferenceData &refdata,
 CORBA::Long max, CORBA::Long min,
 CosRelationships::InterfaceDefs* related_objects_interfaces)
  : RoleFactory_skel (refdata)
{
#if DEBUG
  cout << "  <RoleFactory_impl> RoleFactory_impl "
       << "(max, min, related_objects_interfaces) ..." << flush;
#endif
  first = TRUE;
  InterfaceDefs_nedef = FALSE;
  i_defs = related_objects_interfaces;
  rel_ship_repoid = CORBA::string_dup ("");

#if DEBUG
  cout << "  <RoleFactory_impl> Describe interfaces of related "
       << i_defs->length () << " objects:\n";
  for (CORBA::ULong i=0; i<i_defs->length (); i++) {
    cout << "  <RoleFactory_impl> " << i+1 << ". " << flush;
    CORBA::InterfaceDef::FullInterfaceDescription_var full =
      (*i_defs)[i]->describe_interface ();
    //cout << "Full description:\n";
    //cout << "name: " << full->name << "\n";
    cout << full->id << "\n";
    //cout << "defined_in: " << full->defined_in << "\n";
    //cout << "defined_in: " << full->defined_in << "\n";
    //cout << "version: "<< full->version << "\n";
    //cout << "type code: " << full->type << "\n";
  }
#endif
  max_card = max;
  min_card = min;
#if DEBUG
  cout << "  <RoleFactory_impl> RoleFactory_impl (...) done.\n";
#endif
}



template <class T>
RoleFactory_impl<T>::RoleFactory_impl
(const CORBA::BOA::ReferenceData &refdata,
 CORBA::Long max, CORBA::Long min,
 CosRelationships::InterfaceDefs* related_objects_interfaces,
 char* relship_repoid)
  : RoleFactory_skel (refdata)
{
#if DEBUG
  cout << "  <RoleFactory_impl> RoleFactory_impl "
       << "(max, min, related_objects_interfaces) ..." << flush;
#endif
  first = TRUE;
  InterfaceDefs_nedef = FALSE;
  i_defs = related_objects_interfaces;
  rel_ship_repoid = CORBA::string_dup (relship_repoid);

#if DEBUG
  cout << "  <RoleFactory_impl> Describe interfaces of related "
       << i_defs->length () << " objects:\n";
  for (CORBA::ULong i=0; i<i_defs->length (); i++) {
    cout << "  <RoleFactory_impl> " << i+1 << ". " << flush;
    CORBA::InterfaceDef::FullInterfaceDescription_var full =
      (*i_defs)[i]->describe_interface ();
    //cout << "Full description:\n";
    //cout << "name: " << full->name << "\n";
    cout << full->id << "\n";
    //cout << "defined_in: " << full->defined_in << "\n";
    //cout << "version: "<< full->version << "\n";
    //cout << "type code: " << full->type << "\n";
  }
#endif
  max_card = max;
  min_card = min;

//  return CORBA::InterfaceDef::_duplicate (r_type);
}




template <class T>
RoleFactory_impl<T>::RoleFactory_impl
(const CORBA::BOA::ReferenceData &refdata,
 CORBA::Long max, CORBA::Long min,
 CosRelationships::InterfaceDefs* related_objects_interfaces,
 char* iterator, char* name)
  : RoleFactory_skel (refdata)
{
#if DEBUG
  cout << "  <RoleFactory_impl> RoleFactory_impl "
       << "(tag, max, min, related_objects_interfaces, iterator, name)\n";
#endif
  iterator_server = CORBA::string_dup (iterator);
  name_of_role_server = CORBA::string_dup (name);
  rel_ship_repoid = CORBA::string_dup ("");
  first = TRUE;
  InterfaceDefs_nedef = FALSE;
  i_defs = related_objects_interfaces;

#if DEBUG
  cout << "  <RoleFactory_impl> Describe interfaces of related "
       << i_defs->length () << " objects:\n";
  for (CORBA::ULong i=0; i<i_defs->length (); i++) {
    cout << "  <RoleFactory_impl> " << i+1 << ". " << flush;
    CORBA::InterfaceDef::FullInterfaceDescription_var full =
      (*i_defs)[i]->describe_interface ();
    //cout << "Full description:\n";
    //cout << "name: " << full->name << "\n";
    cout << full->id << "\n";
    //cout << "defined_in: " << full->defined_in << "\n";
    //cout << "defined_in: " << full->defined_in << "\n";
    //cout << "version: "<< full->version << "\n";
    //cout << "type code: " << full->type << "\n";
  }
#endif
  max_card = max;
  min_card = min;
#if DEBUG
  cout << "  <RoleFactory_impl> RoleFactory_impl (...) done.\n";
#endif
}



template <class T>
RoleFactory_impl<T>::RoleFactory_impl
(const CORBA::BOA::ReferenceData &refdata,
 CORBA::Long max, CORBA::Long min,
 CosRelationships::InterfaceDefs* related_objects_interfaces,
 char* iterator, char* relship_repoid, char* name)
  : CosRelationships::RoleFactory_skel (refdata)
{
#if DEBUG
  cout << "  <RoleFactory_impl> RoleFactory_impl "
       << "(tag, max, min, ro_interfaces, relship_repoid, iterator, name)" 
       << flush;
#endif
  iterator_server = CORBA::string_dup (iterator);
  name_of_role_server = CORBA::string_dup (name);
  rel_ship_repoid = CORBA::string_dup (relship_repoid);

  first = TRUE;
  InterfaceDefs_nedef = FALSE;
  i_defs = related_objects_interfaces;

#if DEBUG
  cout << "  <RoleFactory_impl> Describe interfaces of related "
       << i_defs->length () << " objects:\n";
  for (CORBA::ULong i=0; i<i_defs->length (); i++) {
    cout << "  <RoleFactory_impl> " << i+1 << ". " << flush;
    CORBA::InterfaceDef::FullInterfaceDescription_var full =
      (*i_defs)[i]->describe_interface ();
    //cout << "Full description:\n";
    //cout << "name: " << full->name << "\n";
    cout << full->id << "\n";
    //cout << "defined_in: " << full->defined_in << "\n";
    //cout << "version: "<< full->version << "\n";
    //cout << "type code: " << full->type << "\n";
  }
#endif
  max_card = max;
  min_card = min;
}







template <class T>
CORBA::InterfaceDef_ptr
RoleFactory_impl<T>::role_type ()
{
#if DEBUG
  cout << "  <RoleFactory_impl> role_type():\n";
#endif
  CORBA::BOA_var boa = _boa ();
  CORBA::ORB_ptr orb = _orb ();

  if (first) {
#if DEBUG
    cout << "  <RoleFactory_impl> first call...\n";
#endif
      Role_impl* tmp_role
        = create (CORBA::Object::_duplicate (this));
      CosRelationships::Role_ptr role = tmp_role;
      r_type = role->_get_interface ();
      first = FALSE;
#if DEBUG
      cout << "  <RoleFactory_impl> destroy role\n";
#endif
      CORBA::release (role);
  }

#if DEBUG
  cout << "  <RoleFactory_impl> describe interface of role:\n";
  CORBA::InterfaceDef::FullInterfaceDescription_var full_of_role
      = r_type ->describe_interface ();
  cout << "  <RoleFactory_impl> name: " << full_of_role->name << "\n";
  cout << "  <RoleFactory_impl> id: " << full_of_role->id << "\n";
  cout << "  <RoleFactory_impl> defined_in: " << full_of_role->defined_in
       << "\n";
  cout << "  <RoleFactory_impl> version: "<< full_of_role->version << "\n";
#endif

  return CORBA::InterfaceDef::_duplicate (r_type);
}



template <class T>
CORBA::ULong
RoleFactory_impl<T>::max_cardinality ()
{
//  return 2000000000;   // max long - maybe :))
  return max_card;
}


template <class T>
CORBA::ULong
RoleFactory_impl<T>::min_cardinality ()
{
  // return 0;
  return min_card;
}


template <class T>
CosRelationships::InterfaceDefs*
RoleFactory_impl<T>::related_object_types()
{
  if (InterfaceDefs_nedef) {
    CosRelationships::InterfaceDefs*
      tmp = new CosRelationships::InterfaceDefs();
    tmp->length(0);
    return tmp;
  }
  return new CosRelationships::InterfaceDefs (*i_defs);
}

template <class T>
CosRelationships::Role_ptr
RoleFactory_impl<T>::create_role 
(CosRelationships::RelatedObject_ptr related_object)
{

#if DEBUG
  cout << "  <RoleFactory_impl> create_role ()\n";
#endif

#if DEBUG
  if (CORBA::is_nil (related_object))
    cout << "  <RoleFactory_impl> related_object is NIL!\n";
#endif

  if (CORBA::is_nil (related_object)) {
    CosRelationships::RoleFactory::NilRelatedObject exception;
      mico_throw (exception);
  }

  CORBA::Boolean is_in = FALSE;
  char* repoid = CORBA::string_dup (related_object->_repoid ());
#if DEBUG
  cout << "  <RoleFactory_impl> related objects repoid: " << repoid << "\n";
#endif

  for (CORBA::ULong i=0; i<i_defs->length (); i++) {
    CORBA::InterfaceDef::FullInterfaceDescription_var full =
      (*i_defs)[i]->describe_interface ();
#if DEBUG
    cout << "  <RoleFactory_impl> " << i+1 << ". " << full->id << " == "
         << repoid << " : ";
#endif
    if ((*i_defs)[i]->is_a (repoid)) {
      is_in = TRUE;
#if DEBUG
      cout << "YES!\n";
      break;
#endif
    }
#if DEBUG
    cout << "NO!\n";
#endif
  }

  if (!is_in) {
    CosRelationships::RoleFactory::RelatedObjectTypeError exception;
      mico_throw (exception);
  }

  CORBA::BOA_var boa = _boa();
  CORBA::ORB_ptr orb = _orb();

#if DEBUG
  cout << "  <RoleFactory_impl> creating:\n";
#endif

  Role_impl* tmp_role
    = create (CORBA::Object::_duplicate (related_object));
  tmp_role->set_min_cardinality (this->min_cardinality ());
  tmp_role->set_max_cardinality (this->max_cardinality ());
  tmp_role->set_iterator_server (this->iterator_server);
  tmp_role->set_relship_repoid (this->rel_ship_repoid);
  CosRelationships::Role_ptr role = tmp_role;

#if DEBUG
  cout << "  <RoleFactory_impl> role->_repoid (): " << role->_repoid () 
       << "\n";
  cout << "  <RoleFactory_impl> change implementation to: " 
       << name_of_role_server << "\n";
#endif
 
  CORBA::ImplementationDef_var impl = find_impl (name_of_role_server);
  boa->change_implementation (role, impl);

//  boa->deactivate_obj (role);
 
#if DEBUG
  cout << "  <RoleFactory_impl> create_role  done.\n";
  if (CORBA::is_nil (role->related_object ()))
    cout << "  <RoleFactory_impl> role->related_object () is NIL!\n";
#endif
  
  /*
    if (first) {
    r_type = role->_get_interface ();
    first = FALSE;
    }
    */
#if DEBUG
  cout << "  <RoleFactory_impl> create END! with return:\n";
#endif
 
  return CosRelationships::Role::_duplicate (role);

}


template <class T>
Role_impl*
RoleFactory_impl<T>::create (CosRelationships::RelatedObject_ptr obj)
{
#if DEBUG
  cout << "  <RoleFactory_impl> create (obj)\n";
#endif
  return new T (obj, 0);
#if DEBUG
  cout << "  <RoleFactory_impl> create (obj) done.\n";
#endif
}


template<class T>
CORBA::ImplementationDef_ptr 
RoleFactory_impl<T>::find_impl (const char *name)
{
#if DEBUG
  cout << "  <RoleFactory_impl> find_impl ...\n";
#endif
  CORBA::Object_var obj =
      _orbnc()->resolve_initial_references ("ImplementationRepository");
  assert (!CORBA::is_nil (obj));

  CORBA::ImplRepository_var imr = CORBA::ImplRepository::_narrow (obj);
  assert (!CORBA::is_nil (imr));

  CORBA::ImplRepository::ImplDefSeq_var impls = imr->find_by_name (name);
  assert (impls->length() > 0);
  return CORBA::ImplementationDef::_duplicate (impls[(CORBA::ULong)0]);
}


template <class T>
void
RoleFactory_impl<T>::destroy()
{
#if DEBUG
  cout << "  <RoleFactory_impl> RoleFactory destroy: " << counter << "\n";
#endif
  CORBA::BOA_var boa = _boa ();
  CORBA::ORB_var orb = _orb ();
  if (counter == 0) {
#if DEBUG
    cout << "  <RoleFactory_impl> END!\n";
#endif
    boa->deactivate_impl (CORBA::ImplementationDef::_nil());
    orb->shutdown (TRUE);
  }
}



#endif
