/*
 *  Implementation of COSS Relationship Service for MICO
 *  Copyright (C) 1998 Karel Gardas with the assistance of Kay Roemer
 *                                                       & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 *  or to my private e-mail:
 *                 gardask@alpha.inf.upol.cz
 */


#ifndef __Role_impl_h__
#define __Role_impl_h__

#include <mico/CosRelationships.h>

class Role_impl : virtual public CosRelationships::Role_skel
{
  CORBA::Long cardinality;
  CORBA::Long max_cardinality;
  CORBA::Long min_cardinality;
  CosRelationships::RelatedObject_ptr rel_obj;
  CosRelationships::RelationshipHandles rel_ships;
  //CORBA::InterfaceDef_var rel_ship_type;
  char* rel_ship_repoid;
  char* iterator_server; 
  /* it's RelshipsIteratorServer name */
public:
  Role_impl ();
  Role_impl (CORBA::Object_ptr obj);
  Role_impl (CosRelationships::RelatedObject_ptr, CORBA::Long);

  virtual CORBA::Boolean _save_object ();
  virtual CosRelationships::RelatedObject_ptr related_object ();
  virtual CosRelationships::RelatedObject_ptr 
    get_other_related_object (const CosRelationships::RelationshipHandle& rel,
			      const char *target_name);
  virtual CosRelationships::Role_ptr 
    get_other_role (const CosRelationships::RelationshipHandle& rel,
		    const char *target_name);
  virtual void get_relationships (CORBA::ULong how_many,
			 CosRelationships::RelationshipHandles*& rels,
			 CosRelationships::RelationshipIterator_ptr& iterator);
  virtual void destroy_relationships ();
  virtual void destroy ();
  virtual CORBA::Boolean check_minimum_cardinality ();
  virtual void link (const CosRelationships::RelationshipHandle& rel, 
		     const CosRelationships::NamedRoles& named_roles);
  virtual void unlink (const CosRelationships::RelationshipHandle& rel);

  // extension
  
  virtual CORBA::ImplementationDef_ptr find_impl (const char *name);
  virtual void set_min_cardinality (CORBA::Long); 
  virtual void set_max_cardinality (CORBA::Long); 
  virtual void set_cardinality (CORBA::Long); 
  virtual void set_iterator_server (char*);
  virtual void set_relship_repoid (char*);
};


#endif

