// -*- c++ -*-
/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __mico_mico_h__
#define __mico_mico_h__

    
class Buffer {
    enum {
	MINSIZE = 128,
	RESIZE_THRESH = 10000,
	RESIZE_INCREMENT = 10000
    };
    Boolean readonly;
    ULong rptr, wptr;
    ULong ralignbase, walignbase;
    ULong len;
    Octet *buf;

    static Octet *alloc (ULong sz);
    static Octet *realloc (Octet *, ULong osz, ULong nsz);
    static void free (Octet *);
public:
    Buffer (void *);
    Buffer (ULong sz = 0);
    Buffer (const Buffer &);
    ~Buffer ();
    Buffer &operator= (const Buffer &);
    Boolean operator== (const Buffer &);

    void reset (ULong size = MINSIZE);
    void resize (ULong needed);

    Boolean rseek_rel (Long offs)
    {
	assert (rptr + offs <= wptr);
	rptr += offs;
	return TRUE;
    }
    Boolean rseek_beg (ULong offs)
    {
	assert (offs <= wptr);
	rptr = offs;
	return TRUE;
    }
    Boolean rseek_end (ULong offs)
    {
	assert (offs <= len && len - offs <= wptr);
	rptr = len - offs;
	return TRUE;
    }
    ULong ralign_base () const
    {
	return ralignbase;
    }
    void ralign_base (ULong b)
    {
	ralignbase = b;
    }
    ULong rpos () const
    {
	return rptr;
    }

    Boolean ralign (ULong modulo);

    Boolean peek (void *, ULong blen);
    Boolean peek (Octet &);

    Boolean get (void *, ULong blen);
    Boolean get (Octet &);
    Boolean get1 (void *);
    Boolean get2 (void *);
    Boolean get4 (void *);
    Boolean get8 (void *);
    Boolean get16 (void *);

    void wseek_rel (Long offs)
    {
	assert (!readonly);
	assert (wptr + offs >= rptr &&
		wptr + offs <= len);
	wptr += offs;
    }
    void wseek_beg (ULong offs)
    {
	assert (!readonly);
	assert (offs >= rptr &&
		offs <= len);
	wptr = offs;
    }
    void wseek_end (ULong offs)
    {
	assert (!readonly);
	assert (len - offs >= rptr);
	wptr = len - offs;
    }
    ULong walign_base () const
    {
	return walignbase;
    }
    void walign_base (ULong b)
    {
	walignbase = b;
    }
    ULong wpos () const
    {
	assert (!readonly);
	return wptr;
    }

    void walign (ULong modulo);

    void replace (const void *, ULong blen);
    void replace (Octet);

    void put (const void *, ULong blen);
    void put (Octet);
    void put1 (const void *);
    void put2 (const void *);
    void put4 (const void *);
    void put8 (const void *);
    void put16 (const void *);

    ULong length () const
    {
	assert (!readonly);
	return wptr - rptr;
    }
    Octet *data ()
    {
	return &buf[rptr];
    }
    const Octet *data () const
    {
	return &buf[rptr];
    }
    Octet *wdata ()
    {
	return &buf[wptr];
    }
    const Octet *wdata () const
    {
	return &buf[wptr];
    }
    Octet *buffer ()
    {
	return buf;
    }
    const Octet *buffer () const
    {
	return buf;
    }

    void dump (ostream &) const;
};

typedef TVarVar<Buffer> Buffer_var;

#endif // __mico_mico_h__
